from ROOT import TFile
import uproot

from lxml import objectify

import time, cProfile

cacheRootFile = {}

def getFileObj(fileName, usePyRoot):
	global cacheRootFile

	if fileName in cacheRootFile:
		f = cacheRootFile[fileName]
	else:
		if usePyRoot:
			f = TFile(fileName)
		else:
			f = uproot.open(fileName)
		cacheRootFile[fileName] = f
	return f

def getHistoFromFile(fileName, histoName, usePyRoot):
	f = getFileObj(fileName, usePyRoot)

	if usePyRoot:
		histo = f.Get(histoName)
		try:
			histo.GetTitle()
		except:
			print("WARNING! Could not find '" + histoName + "' in '" + fileName + "'")
	else:
		try:
			histo = f[histoName]
		except (KeyError, uproot.deserialization.DeserializationError):
			print("WARNING! Could not find '" + histoName + "' in '" + fileName + "'")	

	return histo

def parseWithPyRoot(dictOfHistos):
	return parseDict(dictOfHistos, True)

def parseWithUpRoot(dictOfHistos):
	return parseDict(dictOfHistos, False)

def parseDict(dictOfHistos, usePyRoot):
	global cacheRootFile
	cacheRootFile = {}
	
	startTime = time.time()
	
	for sample, sampleDict in list(dictOfHistos.items()):
		for histoSysts, histoSystsDict in list(sampleDict.items()):
			print(sample,histoSysts)

			fileUp = histoSystsDict["FileUp"]
			histoUpName = histoSystsDict["HistoUp"]

			fileDown = histoSystsDict["FileDown"]
			histoDownName = histoSystsDict["HistoDown"]

			histoUp = getHistoFromFile(fileUp, histoUpName, usePyRoot)
			histoDown = getHistoFromFile(fileDown, histoDownName, usePyRoot)
		
			if usePyRoot:	
				print("Up",histoUp.Integral())
				print("Down",histoDown.Integral())
			else:
				print("Up",histoUp.to_numpy()[0].tolist())
				print("Down",histoDown.to_numpy()[0].tolist())

	totalTime = int(time.time() - startTime)
	return totalTime

if __name__ == "__main__":
	path = "/lustre/fs22/group/atlas/brueers/tW0LBoosted/histFitterBackup/results/monotop_twmetComb0L1LBoosted_allCRs_normDStoDR_unblind_sigTheo_envelope__pmoder_sig_a250_DM10_H900_tb1_st0p7/NormalMeasurement_CR_0LBoosted_ttbar_cuts.xml"

	basedir = "/lustre/fs22/group/atlas/brueers/tW0LBoosted/histFitterBackup/"

	xmlObj = objectify.parse(path)

	dictOfHistos = {}

	for sample in xmlObj.iter("Sample"):
		sampleName = sample.get("Name")
		dictOfHistos[sampleName] = {}
		for histoSys in sample.iter("HistoSys"):
			SysName = histoSys.get("Name")

			SysPathDown = basedir + histoSys.get("HistoFileLow")
			SysHistoDown = histoSys.get("HistoPathLow") + histoSys.get("HistoNameLow")

			SysPathUp = basedir + histoSys.get("HistoFileHigh")
			SysHistoUp = histoSys.get("HistoPathHigh") + histoSys.get("HistoNameHigh")

			dictOfHistos[sampleName][SysName] = {"FileUp": SysPathUp, "HistoUp": SysHistoUp, "FileDown": SysPathDown, "HistoDown": SysHistoDown}

	profilerPy = cProfile.Profile()
	profilerUp = cProfile.Profile()

	profilerPy.enable()
	tPy = parseWithPyRoot(dictOfHistos)
	profilerPy.disable()
	profilerPy.dump_stats("pyRoot.prof")

	profilerUp.enable()
	tUp = parseWithUpRoot(dictOfHistos)
	profilerUp.disable()
	profilerUp.dump_stats("upRoot.prof")

	print("It took %i seconds to parse with pyRoot" % tPy)
	print("It took %i seconds to parse with upRoot" % tUp)
